@@ -221,8 +221,12 @@ class ADQLComparisonPredicateVisitor(ADQLParserVisitor):
         contains_visitor = ADQLContainsVisitor()
         contains_visitor.visit(ctx)
         if len(contains_visitor.contains):
-            ctx.children[0].removeLastChild()
-            ctx.children[1].removeLastChild()
+            if ctx.children[0].getText().lower()[:8] == 'contains':
+                ctx.children[1].removeLastChild()
+                ctx.children[2].removeLastChild()
+            elif ctx.children[2].getText().lower()[:8] == 'contains':
+                ctx.children[0].removeLastChild()
+                ctx.children[1].removeLastChild()
 
 
 class ADQLFunctionsTranslationVisitor(ADQLParserVisitor):
@@ -391,14 +395,21 @@ class FormatListener(ADQLParserListener):
             nd = self.contexts[node.parentCtx]
         except KeyError:
             nd = node.getText()
+            if isinstance(node.parentCtx,
+                          ADQLParser.Character_string_literalContext):
+                if nd == "'":
+                    nd = None
+                else:
+                    nd = "'" + nd + "'"
 
-        if isinstance(node.parentCtx, ADQLParser.Set_function_typeContext)\
-            or isinstance(node.parentCtx.parentCtx,
-                          ADQLParser.User_defined_function_nameContext)\
-                or nd.upper() in adql_function_names:
-            nd += '_'
+        if nd is not None:
+            if isinstance(node.parentCtx, ADQLParser.Set_function_typeContext)\
+                or isinstance(node.parentCtx.parentCtx,
+                              ADQLParser.User_defined_function_nameContext)\
+                    or nd.upper() in adql_function_names:
+                nd += '_'
 
-        self.nodes.append(nd)
+            self.nodes.append(nd)
 
         try:
             nd = self.limit_contexts[node]
