@@ -11,6 +11,8 @@ from antlr4.error.ErrorListener import ErrorListener
 from .MySQLParser import MySQLParser
 from .MySQLParserListener import MySQLParserListener
 
+#  logging.basicConfig(level=logging.INFO)
+
 
 def parse_alias(alias):
     """
@@ -188,6 +190,14 @@ class ColumnKeywordFunctionListener(MySQLParserListener):
         self.data.append([ctx.depth(), ctx,
                           self._extract_column(ctx, append=False)[1]])
 
+    def enterHaving_clause(self, ctx):
+        self.keywords.append('having')
+        self._extract_column(ctx)
+        logging.info((ctx.depth(), ctx.__class__.__name__,
+                     self._extract_column(ctx, append=False)[1]))
+        self.data.append([ctx.depth(), ctx,
+                          self._extract_column(ctx, append=False)[1]])
+
     def enterOrderby_clause(self, ctx):
         self.keywords.append('order by')
         col = self._extract_column(ctx, append=False)
@@ -250,22 +260,21 @@ class RemoveSubqueriesListener(MySQLParserListener):
     """
     def __init__(self, depth):
         self.depth = depth
-        #  self.subquery_aliases = []
 
     def enterSelect_expression(self, ctx):
         parent = ctx.parentCtx.parentCtx
 
         if isinstance(parent, MySQLParser.SubqueryContext) and ctx.depth() >\
                 self.depth:
-            try:
-                alias = parent.parentCtx.alias()
-            except AttributeError:
-                alias = None
-
-            # subquery alias
-            alias = parse_alias(alias)
-            #  self.subquery_aliases.append([alias, ctx.depth()])
-            ctx.parentCtx.removeLastChild()
+            # we need to remove all Select_expression instances, not
+            # just the last one so we loop over until we get all of them out
+            seinstances = [isinstance(i, MySQLParser.Select_expressionContext)
+                           for i in ctx.parentCtx.children]
+            while True in seinstances:
+                ctx.parentCtx.removeLastChild()
+                seinstances = [isinstance(i,
+                                          MySQLParser.Select_expressionContext)
+                               for i in ctx.parentCtx.children]
 
 
 class SchemaNameListener(MySQLParserListener):
